#include "anthy.h"
#include <QDebug>
#include <QStringList>

#include "romaji.h"

Anthy::Anthy(QObject *parent) : QObject(parent)
{
    m_converting = false;
    m_num = 0;

    if (anthy_init() < 0) {
        qCritical() << "failed anthy_init()";
    }

    if ((ac = anthy_create_context()) == 0) {
        qCritical() << "failed to create anthy context";
    }

    if (ac) {
        anthy_context_set_encoding(ac, ANTHY_UTF8_ENCODING);
    }
}

Anthy::~Anthy()
{
    if (ac) {
        anthy_release_context(ac);
    }
    anthy_quit();
}

QString Anthy::preeditString() const
{
    return m_preedit;
}

void Anthy::append(const QString &str)
{
    // 先頭文字によって挙動を変える
    if (str.isEmpty()) {
        // 何もしない
        qDebug() << "string is empty";
        return;
    } else if (str.at(0) == QChar('\r') || str.at(0) == QChar('\n')) {
        // Enterキーの場合は、バッファーをクリアしてsendCommit()するために何かemit
        qDebug() << "ENTER";
        emit preeditStringCommitted(m_preedit);
        m_preedit = "";
        m_converting = false;
        m_num = 0;
    } else if (str.at(0).isSpace()) {
        // 空白の場合は変換
        qDebug() << "SPACE";
        if (m_preedit.isEmpty()) {
            // そのままコミット
            emit preeditStringCommitted(str);
        } else {
            // 変換処理
            convert();
            ++m_num;
        }
    } else if (str.at(0).isPrint()) {
        // ローマ字変換
        qDebug() << "Roman" << m_preedit << " + " << str;
        if (m_converting) {
            // 前のデータを確定してコミット
            emit preeditStringCommitted(m_preedit);
            m_preedit = "";
            m_converting = false;
            m_num = 0;
        }

        m_preedit += str;

        // 未変換部分を取得しマッチしたら置換
        for (int len = 4; len > 0; --len) {
            QString preroma = m_preedit.right(len);
            QMap<QString, QString>::const_iterator kana = romaji.find(preroma);
            if (kana != romaji.end()) {
                // 右から一回だけreplaceできるといいんだけど
                m_preedit.replace(preroma, kana.value());
                emit preeditStringChanged();
                break;
            }
        }
    }
}

void Anthy::convert()
{
    m_converting = true;

    if (anthy_set_string(ac, m_preedit.toUtf8().constData()) < 0) {
        qWarning() << "failed anthy_set_string()";
        return;
    }

    anthy_conv_stat stat;
    if (anthy_get_stat(ac, &stat) < 0) {
        qWarning() << "failed anthy_get_stat()";
        return;
    }
    qDebug() << "N-seg: " << stat.nr_segment;

    QStringList data;
    char buf[1024];
    for (int i = 0; i < stat.nr_segment; ++i) {
        int len = anthy_get_segment(ac, i, m_num, buf, sizeof(buf));
        if (len == sizeof(buf)) {
            qWarning() << "uncoverted segment overflow";
        }
        qDebug() << "  " << i << ": " << buf;
        data.append(buf);
    }

    m_preedit = data.join("");
    emit preeditStringChanged();
}

